@@ -35,6 +35,7 @@
 #include "alloc.h"
 #include "fatal.h"
 #include "sudo_debug.h"
+#include "sudo_util.h"
 
 #define DEFAULT_TEXT_DOMAIN	"sudo"
 #include "gettext.h"
@@ -90,7 +91,7 @@ aix_setlimits(char *user)
     debug_decl(aix_setlimits, SUDO_DEBUG_UTIL)
 
     if (setuserdb(S_READ) != 0)
-	fatal("unable to open userdb");
+	fatal(U_("unable to open userdb"));
 
     /*
      * For each resource limit, get the soft/hard values for the user
@@ -147,10 +148,10 @@ aix_setauthdb(char *user)
 
     if (user != NULL) {
 	if (setuserdb(S_READ) != 0)
-	    fatal(_("unable to open userdb"));
+	    fatal(U_("unable to open userdb"));
 	if (getuserattr(user, S_REGISTRY, &registry, SEC_CHAR) == 0) {
 	    if (setauthdb(registry, NULL) != 0)
-		fatal(_("unable to switch to registry \"%s\" for %s"),
+		fatal(U_("unable to switch to registry \"%s\" for %s"),
 		    registry, user);
 	}
 	enduserdb();
@@ -167,7 +168,7 @@ aix_restoreauthdb(void)
     debug_decl(aix_setauthdb, SUDO_DEBUG_UTIL)
 
     if (setauthdb(NULL, NULL) != 0)
-	fatal(_("unable to restore registry"));
+	fatal(U_("unable to restore registry"));
 
     debug_return;
 }
