@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2007, 2010 Todd C. Miller <Todd.Miller@courtesan.com>
+ * Copyright (c) 2007, 2010, 2011 Todd C. Miller <Todd.Miller@courtesan.com>
  *
  * Permission to use, copy, modify, and distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -34,8 +34,8 @@ struct lbuf {
 
 void lbuf_init(struct lbuf *, int (*)(const char *), int, const char *, int);
 void lbuf_destroy(struct lbuf *);
-void lbuf_append(struct lbuf *, ...);
-void lbuf_append_quoted(struct lbuf *, const char *, ...);
+void lbuf_append(struct lbuf *, const char *, ...) __printflike(2, 3);
+void lbuf_append_quoted(struct lbuf *, const char *, const char *, ...) __printflike(3, 4);
 void lbuf_print(struct lbuf *);
 
 #endif /* _SUDO_LBUF_H */
