@@ -25,14 +25,11 @@
 #include <def_data.h>
 
 struct list_member {
+    SLIST_ENTRY(list_member) entries;
     char *value;
-    struct list_member *next;
 };
 
-struct def_values {
-    char *sval;		/* string value */
-    int ival;		/* actually an enum */
-};
+SLIST_HEAD(list_members, list_member);
 
 enum list_ops {
     add,
@@ -40,6 +37,12 @@ enum list_ops {
     freeall
 };
 
+/* Mapping of tuple string value to enum def_tuple. */
+struct def_values {
+    char *sval;		/* string value */
+    enum def_tuple nval;/* numeric value */
+};
+
 /*
  * Structure describing compile-time and run-time options.
  */
@@ -52,11 +55,12 @@ struct sudo_defs_types {
     union {
 	int flag;
 	int ival;
+	unsigned int uival;
 	double fval;
 	enum def_tuple tuple;
 	char *str;
 	mode_t mode;
-	struct list_member *list;
+	struct list_members list;
     } sd_un;
 };
 
