@@ -108,16 +108,17 @@ sudoers_setlocale(int newlocale, int *prevlocale)
     return res ? true : false;
 }
 
-static int warning_locale;
-
-void
-warning_set_locale(void)
+#ifdef HAVE_LIBINTL_H
+char *
+warning_gettext(const char *msgid)
 {
+    int warning_locale;
+    char *msg;
+
     sudoers_setlocale(SUDOERS_LOCALE_USER, &warning_locale);
-}
-
-void
-warning_restore_locale(void)
-{
+    msg = gettext(msgid);
     sudoers_setlocale(warning_locale, NULL);
+
+    return msg;
 }
+#endif /* HAVE_LIBINTL_H */
