@@ -59,7 +59,7 @@ struct cache_item_grlist {
 };
 
 struct cache_item *sudo_make_gritem(gid_t gid, const char *group);
-struct cache_item *sudo_make_grlist_item(struct passwd *pw, char * const *groups, char * const *gids);
+struct cache_item *sudo_make_grlist_item(const struct passwd *pw, char * const *groups, char * const *gids);
 struct cache_item *sudo_make_pwitem(uid_t uid, const char *user);
 
 #endif /* _SUDOERS_PWUTIL_H */
