@@ -54,6 +54,7 @@ static int
 check_addr(char *input)
 {
     int expected, matched;
+    const char *errstr;
     size_t len;
     char *cp;
 
@@ -65,7 +66,9 @@ check_addr(char *input)
     cp = input + len;
     while (isspace((unsigned char)*cp))
 	cp++;
-    expected = atoi(cp);
+    expected = strtonum(cp, 0, 1, &errstr);
+    if (errstr != NULL)
+	fatalx("expecting 0 or 1, got %s", cp);
     input[len] = '\0';
 
     matched = addr_matches(input);
@@ -79,7 +82,7 @@ check_addr(char *input)
 static void
 usage(void)
 {
-    fprintf(stderr, "usage: check_addr datafile\n");
+    fprintf(stderr, "usage: %s datafile\n", getprogname());
     exit(1);
 }
 
@@ -91,9 +94,7 @@ main(int argc, char *argv[])
     size_t len;
     FILE *fp;
 
-#if !defined(HAVE_GETPROGNAME) && !defined(HAVE___PROGNAME)
-    setprogname(argc > 0 ? argv[0] : "check_addr");
-#endif
+    initprogname(argc > 0 ? argv[0] : "check_addr");
 
     if (argc != 2)
 	usage();
