@@ -33,13 +33,14 @@
 #include "missing.h"
 #include "fatal.h"
 #include "alloc.h"
-#include "list.h"
+#include "queue.h"
 #include "fileops.h"
 #include "defaults.h"
 #include "logging.h"
 #include "sudo_nss.h"
 #include "sudo_plugin.h"
 #include "sudo_debug.h"
+#include "sudo_util.h"
 
 #define DEFAULT_TEXT_DOMAIN	"sudoers"
 #include "gettext.h"
@@ -268,13 +269,22 @@ int pam_prep_user(struct passwd *);
 
 /* gram.y */
 int sudoersparse(void);
+extern char *login_style;
+extern const char *errorfile;
+extern int errorlineno;
+extern bool parse_error;
+extern bool sudoers_warnings;
 
 /* toke.l */
 YY_DECL;
+extern FILE *sudoersin;
 extern const char *sudoers_file;
+extern char *sudoers;
 extern mode_t sudoers_mode;
 extern uid_t sudoers_uid;
 extern gid_t sudoers_gid;
+extern int sudolineno;
+extern int last_token;
 
 /* defaults.c */
 void dump_defaults(void);
@@ -292,9 +302,9 @@ __dso_public struct group *sudo_getgrgid(gid_t);
 __dso_public struct group *sudo_getgrnam(const char *);
 __dso_public void sudo_gr_addref(struct group *);
 __dso_public void sudo_gr_delref(struct group *);
-bool user_in_group(struct passwd *, const char *);
+bool user_in_group(const struct passwd *, const char *);
 struct group *sudo_fakegrnam(const char *);
-struct group_list *sudo_get_grlist(struct passwd *pw);
+struct group_list *sudo_get_grlist(const struct passwd *pw);
 struct passwd *sudo_fakepwnam(const char *, gid_t);
 struct passwd *sudo_mkpwent(const char *user, uid_t uid, gid_t gid, const char *home, const char *shell);
 struct passwd *sudo_getpwnam(const char *);
@@ -315,12 +325,6 @@ void sudo_setspent(void);
 /* timestr.c */
 char *get_timestr(time_t, int);
 
-/* atobool.c */
-int atobool(const char *str);
-
-/* atoid.c */
-int atoid(const char *str, const char *sep, char **endp, const char **errstr);
-
 /* boottime.c */
 int get_boottime(struct timeval *);
 
@@ -349,9 +353,6 @@ int sudoers_hook_putenv(char *string, void *closure);
 int sudoers_hook_setenv(const char *name, const char *value, int overwrite, void *closure);
 int sudoers_hook_unsetenv(const char *name, void *closure);
 
-/* fmt_string.c */
-char *fmt_string(const char *, const char *);
-
 /* sudoers.c */
 FILE *open_sudoers(const char *, bool, bool *);
 int sudoers_policy_init(void *info, char * const envp[]);
@@ -364,22 +365,12 @@ int sudoers_policy_exec_setup(char *argv[], char *envp[], mode_t cmnd_umask, cha
 extern const char *path_ldap_conf;
 extern const char *path_ldap_secret;
 
-/* aix.c */
-void aix_restoreauthdb(void);
-void aix_setauthdb(char *user);
-
 /* group_plugin.c */
 int group_plugin_load(char *plugin_info);
 void group_plugin_unload(void);
 int group_plugin_query(const char *user, const char *group,
     const struct passwd *pwd);
 
-/* setgroups.c */
-int sudo_setgroups(int ngids, const GETGROUPS_T *gids);
-
-/* gidlist.c */
-int parse_gid_list(const char *gidstr, const gid_t *basegid, GETGROUPS_T **gidsp);
-
 #ifndef _SUDO_MAIN
 extern struct sudo_user sudo_user;
 extern struct passwd *list_pw;
