@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2011 Todd C. Miller <Todd.Miller@courtesan.com>
+ * Copyright (c) 2011-2013 Todd C. Miller <Todd.Miller@courtesan.com>
  *
  * Permission to use, copy, modify, and distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -14,8 +14,8 @@
  * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
 
-#ifndef _SUDO_TOKE_H
-#define _SUDO_TOKE_H
+#ifndef _SUDOERS_TOKE_H
+#define _SUDOERS_TOKE_H
 
 bool append(const char *, int);
 bool fill_args(const char *, int, int);
@@ -23,7 +23,7 @@ bool fill_cmnd(const char *, int);
 bool fill_txt(const char *, int, int);
 bool ipv6_valid(const char *s);
 int sudoers_trace_print(const char *msg);
-void yyerror(const char *);
+void sudoerserror(const char *);
 
 #ifndef FLEX_SCANNER
 extern int (*trace_print)(const char *msg);
@@ -39,4 +39,4 @@ extern int (*trace_print)(const char *msg);
 	(*trace_print)(msg);						\
 } while (0);
 
-#endif /* _SUDO_TOKE_H */
+#endif /* _SUDOERS_TOKE_H */
