@@ -54,7 +54,7 @@ main(int argc, char *argv[], char *envp[])
     textdomain(PACKAGE_NAME);
 
     if (argc < 2)
-	fatalx(_("requires at least one argument"));
+	fatalx(U_("requires at least one argument"));
 
     /* Read sudo.conf. */
     sudo_conf_read(NULL);
@@ -75,7 +75,7 @@ main(int argc, char *argv[], char *envp[])
 	*cp = '-';
     }
     sudo_execve(cmnd, argv, envp, noexec);
-    warning(_("unable to execute %s"), argv[0]);
+    warning(U_("unable to execute %s"), argv[0]);
     sudo_debug_exit_int(__func__, __FILE__, __LINE__, sudo_debug_subsys, EXIT_FAILURE);                
     _exit(EXIT_FAILURE);
 }
